package jdk.xml.internal;

import java.util.MissingResourceException;
import java.util.ListResourceBundle;
import java.util.Locale;
import java.util.ResourceBundle;
import java.io.FilenameFilter;
import java.security.PrivilegedActionException;
import java.io.FileNotFoundException;
import java.security.PrivilegedExceptionAction;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.IOException;
import java.io.File;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.util.Properties;

public class SecuritySupport
{
    public static final String NEWLINE;
    static final Properties cacheProps;
    static volatile boolean firstTime;
    
    private SecuritySupport() {
    }
    
    public static String getSystemProperty(final String propName) {
        return (String)AccessController.doPrivileged(new PrivilegedAction<String>() {
            @Override
            public String run() {
                return System.getProperty(propName);
            }
        });
    }
    
    public static String getSystemProperty(String propName, String defValue) {
        String value = getSystemProperty(propName);
        if (value == null) {
            return defValue;
        }
        return value;
    }
    
    public static <T> T getSystemProperty(Class<T> type, String propName, String defValue) {
        String value = getSystemProperty(propName);
        if (value == null) {
            value = defValue;
        }
        if (Integer.class.isAssignableFrom(type)) {
            return type.cast(Integer.valueOf(Integer.parseInt(value)));
        }
        if (Boolean.class.isAssignableFrom(type)) {
            return type.cast(Boolean.valueOf(Boolean.parseBoolean(value)));
        }
        return type.cast(value);
    }
    
    public static <T> T getJAXPSystemProperty(Class<T> type, String propName, String defValue) {
        String value = getJAXPSystemProperty(propName);
        if (value == null) {
            value = defValue;
        }
        if (Integer.class.isAssignableFrom(type)) {
            return type.cast(Integer.valueOf(Integer.parseInt(value)));
        }
        if (Boolean.class.isAssignableFrom(type)) {
            return type.cast(Boolean.valueOf(Boolean.parseBoolean(value)));
        }
        return type.cast(value);
    }
    
    public static String getJAXPSystemProperty(String propName) {
        String value = getSystemProperty(propName);
        if (value == null) {
            value = readJAXPProperty(propName);
            return value;
        }
        return value;
    }
    
    public static String readJAXPProperty(String propName) {
        String value = null;
        InputStream is = null;
        try {
            if (SecuritySupport.firstTime) {
                synchronized (SecuritySupport.cacheProps) {
                    if (SecuritySupport.firstTime) {
                        String configFile = getSystemProperty("java.home") + File.separator + "lib" + File.separator + "jaxp.properties";
                        File f = new File(configFile);
                        if (isFileExists(f)) {
                            is = getFileInputStream(f);
                            SecuritySupport.cacheProps.load(is);
                        }
                        SecuritySupport.firstTime = false;
                    }
                }
            }
            value = SecuritySupport.cacheProps.getProperty(propName);
        }
        catch (IOException ex) {}
        finally {
            if (is != null) {
                try {
                    is.close();
                }
                catch (IOException ex2) {}
            }
        }
        return value;
    }
    
    public static boolean isDirectory(final File f) {
        return (boolean)AccessController.doPrivileged(new PrivilegedAction() {
            @Override
            public Object run() {
                return f.isDirectory() ? Boolean.TRUE : Boolean.FALSE;
            }
        });
    }
    
    public static boolean isFileExists(final File f) {
        return (boolean)AccessController.doPrivileged(new PrivilegedAction() {
            @Override
            public Object run() {
                return f.exists() ? Boolean.TRUE : Boolean.FALSE;
            }
        });
    }
    
    public static FileInputStream getFileInputStream(final File file) throws FileNotFoundException {
        try {
            return (FileInputStream)AccessController.doPrivileged(new PrivilegedExceptionAction<FileInputStream>() {
                @Override
                public FileInputStream run() throws Exception {
                    return new FileInputStream(file);
                }
            });
        }
        catch (PrivilegedActionException e) {
            throw (FileNotFoundException)e.getException();
        }
    }
    
    public static String[] getFileList(final File f, final FilenameFilter filter) {
        return (String[])(String[])AccessController.doPrivileged(new PrivilegedAction() {
            @Override
            public Object run() {
                return f.list(filter);
            }
        });
    }
    
    public static ResourceBundle getResourceBundle(String bundle) {
        return getResourceBundle(bundle, Locale.getDefault());
    }
    
    public static ListResourceBundle getResourceBundle(final String bundle, final Locale locale) {
        return (ListResourceBundle)AccessController.doPrivileged(new PrivilegedAction<ListResourceBundle>() {
            @Override
            public ListResourceBundle run() {
                try {
                    return (ListResourceBundle)ResourceBundle.getBundle(bundle, locale);
                }
                catch (MissingResourceException e) {
                    try {
                        return (ListResourceBundle)ResourceBundle.getBundle(bundle, new Locale("en", "US"));
                    }
                    catch (MissingResourceException e2) {
                        throw new MissingResourceException("Could not load any resource bundle by " + bundle, bundle, "");
                    }
                }
            }
        });
    }
    
    static {
        NEWLINE = getSystemProperty("line.separator", "\n");
        cacheProps = new Properties();
        SecuritySupport.firstTime = true;
    }
}
